package engine

func (engine *Engine) NumTokenIndexAdded() uint64 {
	return engine.numTokenIndexAdded
}

func (engine *Engine) NumDocumentsIndexed() uint64 {
	return engine.numDocumentsIndexed
}

func (engine *Engine) NumDocumentsRemoved() uint64 {
	return engine.numDocumentsRemoved
}

func (engine *Engine) NumDocumentsForceUpdated() uint64 {
	return engine.numDocumentsForceUpdated
}

func (engine *Engine) NumIndexingRequests() uint64 {
	return engine.numIndexingRequests
}

func (engine *Engine) NumRemovingRequests() uint64 {
	return engine.numRemovingRequests
}

func (engine *Engine) NumForceUpdatingRequests() uint64 {
	return engine.numForceUpdatingRequests
}

func (engine *Engine) NumDocumentsStored() uint64 {
	return engine.numDocumentsStored
}
